@@ -1,7 +1,7 @@
 package Perinci::Sub::Wrapper;
 
-our $DATE = '2015-01-16'; # DATE
-our $VERSION = '0.69'; # VERSION
+our $DATE = '2015-01-17'; # DATE
+our $VERSION = '0.70'; # VERSION
 
 use 5.010001;
 use strict;
@@ -581,61 +581,90 @@ sub _handle_args {
                 $self->indent;
 
                 if ($argspec->{stream}) {
-                    $self->_add_module('Scalar::Util');
                     $self->_errif(
-                        500,
-                        qq["Argument '$prefix$argname' (stream) fails validation:  must be either a glob, object that supports getitem/getline, or a (tied) array"],
-                        "!(ref($argterm) eq 'GLOB' || (Scalar::Util::blessed($argterm) && ($argterm->can('getitem') || $argterm->can('getline')) || ref($argterm) eq 'ARRAY'))",
+                        400,
+                        qq["Argument '$prefix$argname' (stream) fails validation: must be coderef"],
+                        "!(ref($argterm) eq 'CODE')",
                     );
+                    $self->push_lines('{ # introduce scope because we want to declare a generic variable $i');
+                    $self->indent;
+                    $self->push_lines(
+                        'my $i = -1;',
+                        "my \$origsub = $argterm;",
+                        '# arg coderef wrapper for validation',
+                        "$argterm = sub {",
+                    );
+                    $self->indent;
+                    $self->push_lines(
+                        '$i++;',
+                        "my \$rec = \$origsub->();",
+                        'return undef unless defined $rec;',
+                    );
+                }
 
-                } else {
-                    my $dn = $argname; $dn =~ s/\W+/_/g;
-                    my $cd = $self->_plc->compile(
-                        data_name            => $dn,
-                        data_term            => $argterm,
-                        schema               => $sch,
-                        schema_is_normalized => $opt_sin,
-                        return_type          => 'str',
-                        indent_level         => $self->get_indent_level + 4,
-                        %{ $self->{_args}{_extra_sah_compiler_args} // {}},
+                my $dn = $argname; $dn =~ s/\W+/_/g;
+                my $cd = $self->_plc->compile(
+                    data_name            => $dn,
+                    data_term            => $argspec->{stream} ? '$rec' : $argterm,
+                    schema               => $sch,
+                    schema_is_normalized => $opt_sin,
+                    return_type          => 'str',
+                    indent_level         => $self->get_indent_level + 1,
+                    %{ $self->{_args}{_extra_sah_compiler_args} // {}},
+                );
+                $self->_add_module($_) for @{ $cd->{modules} };
+                $self->_add_var($_, $cd->{vars}{$_})
+                    for sort keys %{ $cd->{vars} };
+                $cd->{result} =~ s/\A\s+//;
+                $self->push_lines(
+                    "my \$err_$dn;",
+                    "$cd->{result};",
+                );
+                if ($argspec->{stream}) {
+                    $self->push_lines(
+                        'if ('."\$err_$dn".') { die "Record #$i of streaming argument '."'$prefix$argname'".' fails validation: '."\$err_$dn".'" }',
+                        '$rec;',
                     );
-                    $self->_add_module($_) for @{ $cd->{modules} };
-                    $self->_add_var($_, $cd->{vars}{$_})
-                        for sort keys %{ $cd->{vars} };
-                    $self->push_lines("my \$err_$dn;\n$cd->{result};");
+                } else {
                     $self->_errif(
                         400, qq["Argument '$prefix$argname' fails validation: \$err_$dn"],
                         "\$err_$dn");
-                    if ($argspec->{meta}) {
-                        $self->push_lines("# check subargs of $prefix$argname");
-                        $self->_handle_args(
+                }
+                if ($argspec->{meta}) {
+                    $self->push_lines("# check subargs of $prefix$argname");
+                    $self->_handle_args(
                             %args,
                             v => $argspec->{meta}{args},
                             prefix => ($prefix ? "$prefix/" : "") . "$argname/",
                             argsterm => '%{'.$argterm.'}',
                         );
-                    }
-                    if ($argspec->{element_meta}) {
-                        $self->push_lines("# check element subargs of $prefix$argname");
-                        my $indexterm = "$prefix$argname";
-                        $indexterm =~ s/\W+/_/g;
-                        $indexterm = '$i_' . $indexterm;
-                        $self->push_lines('for my '.$indexterm.' (0..$#{ '.$argterm.' }) {');
-                        $self->indent;
-                        $self->_errif(
-                            400, qq("Argument '$prefix$argname\[).qq($indexterm]' fails validation: must be hash"),
-                            "ref($argterm\->[$indexterm]) ne 'HASH'");
-                        $self->_handle_args(
-                            %args,
-                            v => $argspec->{element_meta}{args},
-                            prefix => ($prefix ? "$prefix/" : "") . "$argname\[$indexterm]/",
-                            argsterm => '%{'.$argterm.'->['.$indexterm.']}',
-                        );
-                        $self->unindent;
-                        $self->push_lines('}'); # if exists arg
-                    }
+                }
+                if ($argspec->{element_meta}) {
+                    $self->push_lines("# check element subargs of $prefix$argname");
+                    my $indexterm = "$prefix$argname";
+                    $indexterm =~ s/\W+/_/g;
+                    $indexterm = '$i_' . $indexterm;
+                    $self->push_lines('for my '.$indexterm.' (0..$#{ '.$argterm.' }) {');
+                    $self->indent;
+                    $self->_errif(
+                        400, qq("Argument '$prefix$argname\[).qq($indexterm]' fails validation: must be hash"),
+                        "ref($argterm\->[$indexterm]) ne 'HASH'");
+                    $self->_handle_args(
+                        %args,
+                        v => $argspec->{element_meta}{args},
+                        prefix => ($prefix ? "$prefix/" : "") . "$argname\[$indexterm]/",
+                        argsterm => '%{'.$argterm.'->['.$indexterm.']}',
+                    );
+                    $self->unindent;
+                    $self->push_lines('}');
                 }
                 $self->unindent;
+                if ($argspec->{stream}) {
+                    $self->push_lines('}; # arg coderef wrapper');
+                    $self->unindent;
+                    $self->push_lines('} # close scope');
+                    $self->unindent;
+                }
                 if ($has_default_prop) {
                     $self->push_lines(
                         '} else {',
@@ -645,7 +674,7 @@ sub _handle_args {
                         '} else {',
                         "    $argterm //= ".__squote($sch->[1]{default}).";");
                 }
-                $self->push_lines('}');
+                $self->push_lines("} # if exists arg $prefix$argname");
             } # if opt_va
 
         } elsif ($has_default_prop) {
@@ -744,25 +773,24 @@ sub handle_result {
 
     my $sub_name = $self->{_args}{sub_name};
 
-    if ($v->{stream}) {
-
+    if ($opt_vr) {
         $self->select_section('after_call_res_validation');
-        $self->push_lines("my \$_w_res2 = \$_w_res->[2];");
-        $self->_add_module('Scalar::Util');
+        $self->push_lines(
+            'my $_w_res2 = $_w_res->[2];',
+            'my $_w_res_is_stream = $_w_res->[3]{stream} // ' . ($v->{stream} ? 1:0) . ';',
+        );
         $self->_errif(
             500,
-            q["Stream result must be either a glob, object that supports getitem/getline, or a (tied) array"],
-            '!(ref($_w_res2) eq "GLOB" || (Scalar::Util::blessed($_w_res2) && ($_w_res2->can("getitem") || $_w_res2->can("getline")) || ref($_w_res2) eq "ARRAY"))',
+            q["Stream result must be coderef"],
+            '$_w_res_is_stream && ref($_w_res2) ne "CODE"',
         );
-
-    } elsif (keys %schemas_by_status) {
-
-        $self->select_section('after_call_res_validation');
-        $self->push_lines("my \$_w_res2 = \$_w_res->[2];");
-        $self->push_lines("my \$_w_err2_res;");
-
         for my $s (sort keys %schemas_by_status) {
             my $sch = $schemas_by_status{$s};
+            $self->push_lines("if (\$_w_res->[0] == $s) {");
+            $self->indent;
+            $self->push_lines('if (!$_w_res_is_stream) {');
+            $self->indent;
+
             my $cd = $self->_plc->compile(
                 data_name            => '_w_res2',
                 # err_res can clash on arg named 'res'
@@ -770,14 +798,14 @@ sub handle_result {
                 schema               => $sch,
                 schema_is_normalized => $opt_sin,
                 return_type          => 'str',
-                indent_level         => $self->get_indent_level + 4,
+                indent_level         => $self->get_indent_level + 1,
                 %{ $self->{_args}{_extra_sah_compiler_args} // {}},
             );
             $self->_add_module($_) for @{ $cd->{modules} };
             $self->_add_var($_, $cd->{vars}{$_})
                 for sort keys %{ $cd->{vars} };
-            $self->push_lines("if (\$_w_res->[0] == $s) {");
-            $self->indent;
+            $self->push_lines("my \$_w_err2_res;");
+            $cd->{result} =~ s/\A\s+//;
             $self->push_lines("$cd->{result};");
             $self->_errif(
                 500,
@@ -785,9 +813,42 @@ sub handle_result {
                     qq[\$_w_err2_res"],
                 "\$_w_err2_res");
             $self->unindent;
-            $self->push_lines("}");
+            $self->push_lines("} else {"); # stream
+            $self->indent;
+            $self->push_lines(
+                'my $i = -1;',
+                '# wrap result coderef for validation',
+                '$_w_res->[2] = sub {',
+            );
+            $self->indent;
+            $self->push_lines(
+                '$i++;',
+                'my $rec = $_w_res2->();',
+                'return undef unless defined $rec;',
+            );
+            # generate schema code once again, this time for when stream
+            $cd = $self->_plc->compile(
+                data_name            => 'rec',
+                # err_res can clash on arg named 'res'
+                err_term             => '$rec_err',
+                schema               => $sch,
+                schema_is_normalized => $opt_sin,
+                return_type          => 'str',
+                indent_level         => $self->get_indent_level + 1,
+                %{ $self->{_args}{_extra_sah_compiler_args} // {}},
+            );
+            $self->push_lines('my $rec_err;');
+            $cd->{result} =~ s/\A\s+//;
+            $self->push_lines("$cd->{result};");
+            $self->push_lines('if ($rec_err) { die "BUG: Result stream record #$i fail validation: $rec_err" }');
+            $self->push_lines('$rec;');
+            $self->unindent;
+            $self->push_lines('}; # result coderef wrapper');
+            $self->unindent;
+            $self->push_lines("} # if stream");
+            $self->unindent;
+            $self->push_lines("} # if status=$s");
         } # for schemas_by_status
-
     }
 }
 
@@ -1278,7 +1339,7 @@ Perinci::Sub::Wrapper - A multi-purpose subroutine wrapping framework
 
 =head1 VERSION
 
-This document describes version 0.69 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-01-16.
+This document describes version 0.70 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-01-17.
 
 =head1 SYNOPSIS
 
@@ -1365,7 +1426,7 @@ wrapping code to source code during build.
 
 =head2 wrap_sub(%args) -> [status, msg, result, meta]
 
-{en_US Wrap subroutine to do various things, like enforcing Rinci properties}.
+Wrap subroutine to do various things, like enforcing Rinci properties.
 
 Arguments ('*' denotes required arguments):
 
@@ -1373,87 +1434,71 @@ Arguments ('*' denotes required arguments):
 
 =item * B<compile> => I<bool> (default: 1)
 
-{en_US Whether to compile the generated wrapper}.
+Whether to compile the generated wrapper.
 
-{en_US 
 Can be set to 0 to not actually wrap but just return the generated wrapper
 source code.
-}
 
 =item * B<convert> => I<hash>
 
-{en_US Properties to convert to new value}.
+Properties to convert to new value.
 
-{en_US 
 Not all properties can be converted, but these are a partial list of those that
 can: v (usually do not need to be specified when converting from 1.0 to 1.1,
 will be done automatically), args_as, result_naked, default_lang.
-}
 
 =item * B<debug> => I<bool> (default: 0)
 
-{en_US Generate code with debugging}.
+Generate code with debugging.
 
-{en_US 
 If turned on, will produce various debugging in the generated code. Currently
 what this does:
 
 =over
 
 =item * add more comments (e.g. for each property handler)
-}
 
 =back
 
 =item * B<meta>* => I<hash>
 
-{en_US The function metadata}.
+The function metadata.
 
 =item * B<meta_name> => I<str>
 
-{en_US Where to find the metadata, e.g. "$SPEC{foo}"}.
+Where to find the metadata, e.g. "$SPEC{foo}".
 
-{en_US 
 Some wrapper code (e.g. handler for C<dep> property) needs to refer to the
 function metadata. If not provided, the wrapper will store the function metadata
 in a unique variable (e.g. C<$Perinci::Sub::Wrapped::meta34127816>).
-}
 
 =item * B<sub> => I<str>
 
-{en_US The code to be wrapped}.
+The code to be wrapped.
 
-{en_US 
 At least one of C<sub> or C<sub_name> must be specified.
-}
 
 =item * B<sub_name> => I<str>
 
-{en_US The name of the subroutine, e.g. func or Foo::func (qualified)}.
+The name of the subroutine, e.g. func or Foo::func (qualified).
 
-{en_US 
 At least one of C<sub> or C<sub_name> must be specified.
-}
 
 =item * B<validate_args> => I<bool>
 
-{en_US Whether wrapper should validate arguments}.
+Whether wrapper should validate arguments.
 
-{en_US 
 If set to true, will validate arguments. Validation error will cause status 400
 to be returned. The default is to enable this unless previous wrapper(s) have
 already done this.
-}
 
 =item * B<validate_result> => I<bool>
 
-{en_US Whether wrapper should validate arguments}.
+Whether wrapper should validate arguments.
 
-{en_US 
 If set to true, will validate sub's result. Validation error will cause wrapper
 to return status 500 instead of sub's result. The default is to enable this
 unless previous wrapper(s) have already done this.
-}
 
 =back
 
@@ -1466,10 +1511,9 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
-Return value: {en_US The wrapped subroutine along with its new metadata} (hash)
+Return value: The wrapped subroutine along with its new metadata (hash)
 
 
-{en_US 
 Aside from wrapping the subroutine, the wrapper will also create a new metadata
 for the subroutine. The new metadata is a clone of the original, with some
 properties changed, e.g. schema in C<args> and C<result> normalized, some values
@@ -1479,7 +1523,6 @@ The new metadata will also contain (or append) the wrapping log located in the
 C<x.perinci.sub.wrapper.logs> attribute. The wrapping log marks that the wrapper
 has added some functionality (like validating arguments or result) so that
 future nested wrapper can choose to avoid duplicating the same functionality.
-}
 
 =for Pod::Coverage ^(new|handle(meta)?_.+|wrap|add_.+|section_empty|indent|unindent|get_indent_level|select_section|push_lines)$
 
@@ -1652,7 +1695,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Su
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-Sub-Wrapper>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-Sub-Wrapper>.
 
 =head1 BUGS
 
