@@ -1,24 +1,28 @@
 package Test::Perinci::Sub::Wrapper;
 
+our $DATE = '2015-01-17'; # DATE
+our $VERSION = '0.70'; # VERSION
+
 use 5.010;
 use strict;
 use warnings;
 
 use Function::Fallback::CoreOrPP qw(clone);
+#use List::Util qw(shuffle);
 use Perinci::Sub::Wrapper qw(wrap_sub);
 use Test::More 0.96;
 
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(test_wrap);
 
-our $VERSION = '0.69'; # VERSION
-
 sub test_wrap {
     my %test_args = @_;
     $test_args{wrap_args} or die "BUG: wrap_args not defined";
     my $test_name = $test_args{name} or die "BUG: test_name not defined";
 
     for my $wrapper_type (qw/dynamic embed/) {
+        next if $wrapper_type eq 'dynamic' && $test_args{skip_dynamic};
+        next if $wrapper_type eq 'embed'   && $test_args{skip_embed};
         subtest "$test_name ($wrapper_type)" => sub {
 
             if ($test_args{pretest}) {
@@ -170,13 +174,19 @@ sub test_wrap {
                         }
 
                         if (exists $call->{res}) {
+                            diag explain $call;
                             is_deeply($res, $call->{res}, "res")
                                 or diag explain $res;
                         }
 
+                        if (exists $call->{actual_res}) {
+                            is_deeply($res->[2], $call->{actual_res}, "actual res")
+                                or diag explain $res->[2];
+                        }
+
                         if (exists $call->{actual_res_re}) {
                             like($res->[2], $call->{actual_res_re},
-                                 "actual res");
+                                 "actual res re");
                         }
                     }; # subtest call #$i
                 }
@@ -207,7 +217,7 @@ Test::Perinci::Sub::Wrapper - Provide test_wrap() to test wrapper
 
 =head1 VERSION
 
-This document describes version 0.69 of Test::Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-01-16.
+This document describes version 0.70 of Test::Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-01-17.
 
 =for Pod::Coverage test_wrap
 
@@ -217,7 +227,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Su
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Perinci-Sub-Wrapper>.
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-Sub-Wrapper>.
 
 =head1 BUGS
 
